init -999 python:
    #Fix modules
    renpy.add_python_directory('mods/compatibility/python-packages')
init python:
    #MOD DOWNLOADER
    class ModDownloader():
        import json
        import certifi
        import requests
        import os
        import shutil
        import zipfile
        import time

        def __init__(self):
            self.android = renpy.android
            self.url_json = "https://files.lessonsinlove.club/packages/en/list.json"
            self.headers = {"User-Agent": "BallastModInstaller"}
            if self.android:
                self.mods_path = self.os.path.join(config.savedir, "game", "mods")
            else:
                self.mods_path = self.os.path.join(config.basedir, "game", "mods")
            self.json_path = self.os.path.join(self.mods_path, "compatibility", "list.json")
            #Repair certifi
            self.os.environ["REQUESTS_CA_BUNDLE"] = self.ca_file()
            # self.certifi.core.where = self.ca_file
            self.certifi.where = self.ca_file
            self.requests.utils.DEFAULT_CA_BUNDLE_PATH = self.ca_file()
            self.requests.adapters.DEFAULT_CA_BUNDLE_PATH = self.ca_file()

        #Repair certifi
        def ca_file(self):
            if not self.android:
                file = self.os.path.join(config.basedir, "game", "python-packages", "certifi", "cacert.pem")
            else:
                file = self.os.path.join(self.mods_path, "compatibility", "cacert.pem")
                if not self.os.path.exists(file):
                    with open(file, "wb") as f:
                        f.write(renpy.file("python-packages/certifi/cacert.pem").read())
            
            return file

        def version_analizer(self, version1, version2):
            version1 = version1.split(".")

            version2 = version2.split(".")

            for idx, n in enumerate(version1):
                if int(n) >= int(version2[idx]):
                    continue
                else:
                    return "OLD"

            return "NEW"

        def check_mods(self, init_phase=False):

            persistent.mod_updater.clear()

            persistent.mod_updater = {
                "MODS": {},
                "INSTALLED_MODS": {},
                "TOGGLE_MODS": {},
                "DELETE": True,
                "STATUS": '',
                "PROGRESS": 0,
                "LENGH": "0/0",
                "DOWNLOADING_MOD": '',
                "LAST_UPDATE": ''
            }

            try:
                data = self.requests.get(self.url_json, headers=self.headers, timeout=1.25)
            except:
                if not init_phase:
                    renpy.show_screen("dialog", message="Failed to connect to server")
                    renpy.restart_interaction()
                return

            with open(self.json_path, 'wb') as f:
                f.write(data.content)

            data = data.json()        
            
            persistent.mod_updater["MODS"] = data["packages"]

            for mod in persistent.mod_updater["MODS"]:
                if config.version in persistent.mod_updater["MODS"][mod]["versions"]:
                    if self.os.path.exists(self.os.path.join(self.mods_path, mod)):
                        
                        file = self.os.path.join(self.mods_path, mod, "mod.json")

                        if self.os.path.exists(file):
                            ver = self.json.load(open(self.os.path.join(self.mods_path, mod, "mod.json")))["version"]

                            persistent.mod_updater["INSTALLED_MODS"][mod] = self.version_analizer(ver, data["packages"][mod]["versions"][config.version])
                        else:
                            persistent.mod_updater["INSTALLED_MODS"][mod] = "UNCORRECT"
                    else:
                        persistent.mod_updater["INSTALLED_MODS"][mod] = "NOT_INSTALLED"
                else:
                    persistent.mod_updater["INSTALLED_MODS"][mod] = "VERSION_ERROR"
                persistent.mod_updater["TOGGLE_MODS"][mod] = False
                
            persistent.mod_updater["DOWNLOADING_MOD"] = ''

            if not init_phase:
                renpy.show_screen("mod_news")
            renpy.restart_interaction()

            return
            
        def convert_bytes(self, size, full_size):
            # 2**10 = 1024
            power = 2**10
            n1 = 0
            n2 = 0
            power_labels = {0 : 'B', 1: 'Kb', 2: 'Mb', 3: 'Gb', 4: 'Tb'}
            while size > power:
                size /= power
                n1 += 1
            while full_size > power:
                full_size /= power
                n2 += 1
            return "%s%s/%s%s" % (int(size), power_labels[n1], int(full_size), power_labels[n2])

        def download_mod(self, session, url):
            persistent.mod_updater["PROGRESS"] = 0
            persistent.mod_updater["STATUS"] = "DOWNLOADING"
            file_name = self.os.path.join(self.mods_path, url.split('/')[-1])

            with open(file_name, "wb") as f:
                with session.get(url, headers=self.headers, stream=True, timeout=1.25) as http:
                    total_length = int(http.headers.get('Content-Length'))
                    dl = 0
                    for chunk in http.iter_content(1024 if total_length < 4*1024*1024 else 1024*1024):
                        dl += len(chunk)
                        persistent.mod_updater["LENGH"] = self.convert_bytes(dl, total_length)
                        f.write(chunk)
                        persistent.mod_updater["PROGRESS"] = int(100 * dl / total_length)
            return file_name

        def save_your_files(self, mod, old_path, path):
            if not mod in save_mod_dir:
                return
            persistent.mod_updater["STATUS"] = "SAVE_FILES"
            for dir in save_mod_dir[mod]:
                if self.os.path.exists(self.os.path.join(old_path, dir)):
                    self.os.rename(self.os.path.join(old_path, dir), self.os.path.join(path, dir))
            return
        def delete(self, name):
            persistent.mod_updater["STATUS"] = "DELETING"
            if not self.os.path.exists(name):
                return
            if self.os.path.isfile(name):
                self.os.remove(name)
            if self.os.path.isdir(name):
                self.shutil.rmtree(name)
            return

        def extract(self, file_name):
            persistent.mod_updater["STATUS"] = "EXTRACTING"
            with self.zipfile.ZipFile(file_name) as f:
                f.extractall(self.mods_path)
            return

        def start_install(self):
            if self.android:
                persistent.mod_updater["STATUS"] = "ANDROID"
                self.time.sleep(3)
            persistent.mod_updater["STATUS"] = "START"
            self.time.sleep(0.5)
            try:
                renpy.set_autoreload(False)
                session = self.requests.Session()
                for mod, value in persistent.mod_updater["TOGGLE_MODS"].items():
                    if value:
                        persistent.mod_updater["DOWNLOADING_MOD"] = mod
                        if config.version in persistent.mod_updater["MODS"][mod]["versions"]:
                            url = "https://files.lessonsinlove.club/packages/en/%s/%s-v%s.zip" % (mod, mod, persistent.mod_updater["MODS"][mod]["versions"][config.version])
                            file_name = self.download_mod(session, url)
                            path = self.os.path.join(self.mods_path, mod)
                            path_old = path + "_old"
                            if self.os.path.exists(path):
                                self.os.rename(path, path_old)
                            self.extract(file_name)
                            if not persistent.mod_updater["DELETE"]:
                                self.save_your_files(mod, path_old, path)
                            self.delete(path_old)
                            self.delete(file_name)
                        else:
                            persistent.mod_updater["STATUS"] = "VERSION_ERROR"
                            self.time.sleep(1)
                persistent.mod_updater["STATUS"] = "FINISH"
                return
            except Exception as exception:
                print(exception)
                persistent.mod_updater["STATUS"] = "ERROR"
                return
        
        def thread(self):
            renpy.show_screen("download_screen")
            renpy.invoke_in_thread(self.start_install)
            renpy.restart_interaction()
            return

        def text_gen(self, *args):
            string = ''
            for text in args:
                if isinstance(text, tuple):
                    for mod, value in text[1].items():
                        for path in value:
                            string = "%s%s%s/%s%s\n" % (string, text[0], mod, path, text[2])
                else:
                    string = "%s%s\n" % (string, text)
            return string

        def start(self):
            for _, value in persistent.mod_updater["TOGGLE_MODS"].items():
                if value:
                    renpy.show_screen("yes_no")
                    renpy.restart_interaction()
                    return
                else:
                    continue
            renpy.show_screen("dialog", message="You didn't choose anything\n    {size=+40}{b}-_-{/b}{/size}")
            renpy.restart_interaction()

        def news(self):
            try:
                self.check_mods(init_phase=True)
                for mod in persistent.mod_updater["INSTALLED_MODS"]:
                    if not persistent.mod_updater["INSTALLED_MODS"][mod] in ["NEW", "NOT_INSTALLED", "VERSION_ERROR"]:
                        renpy.show_screen("dialog", message="Modification update available!", ok_action=Show("mod_news"))
                        renpy.restart_interaction()
                        return
                return
            except Exception as exception:
                print(exception)
                return

        def CheckMods(self):
            return CheckMods(self)

        def StartInstall(self):
            return StartInstall(self)

    md = ModDownloader()

    @renpy.pure
    class CheckMods(Action, FieldEquality):
        identity_fields = [ "downloader" ]
        def __init__(self, downloader):
            self.downloader = downloader
        
        def __call__(self):
            return self.downloader.check_mods()
        
    @renpy.pure
    class StartInstall(Action, FieldEquality):
        identity_fields = [ "downloader" ]
        def __init__(self, downloader):
            self.downloader = downloader
        
        def __call__(self):
            return self.downloader.start()
        
    @renpy.pure
    class SelectModButton(Action, FieldEquality):
        def __init__(self, mod):
            self.mod = mod
        def __call__(self):
            persistent.mod_updater["TOGGLE_MODS"][self.mod] = not persistent.mod_updater["TOGGLE_MODS"][self.mod]
            renpy.restart_interaction()
        def get_sensitive(self):
            if config.version in persistent.mod_updater["MODS"][self.mod]["versions"]:
                return True
            else:
                return False
        def get_selected(self):
            if persistent.mod_updater["TOGGLE_MODS"][self.mod]:
                return True
            else:
                return False
    def SelectMod(mod):
        return SelectModButton(mod)

label splashscreen:
    $ md.news()
    return

init python:
    #last label
    def label_callback(name, abnormal):
        if name.startswith("_") or name in ['after_load', 'main_menu_screen', 'save_screen', 'splashscreen']: return
        store.last_label = name 

    config.label_callback = label_callback

    #discord rpc
    def DiscordRPC():
        import re as regular
        callbacks = {
            'ready': readyCallback,
            'disconnected': disconnectedCallback,
            'error': errorCallback,
        }
        start = time.time()
        while True:
            presence = {
                'start_timestamp': start,
                'large_image_key': 'base'
            }
            try:
                if persistent.discord_rpc_on:
                    if main_menu:
                        presence["details"] = "In main menu"
                    elif persistent.discord_rpc_label_on and globals().get("last_label"):
                        presence["details"] = "Label: %s" % store.last_label
                    
                    if not main_menu and persistent.discord_rpc_say_on and renpy.store._last_say_what is not None:
                        if renpy.store._last_say_who is not None:
                            presence["state"] = regular.sub(r"\{[^{}]+\}", '', globals()[renpy.store._last_say_who].name + ': ' +  renpy.store._last_say_what)[:50].encode('utf-8')
                        else:
                            presence["state"] = regular.sub(r"\{[^{}]+\}", '', renpy.store._last_say_what)[:50].encode('utf-8')
                        if len(presence["state"]) <= 2:
                            presence["state"] += "///"
                        
                    discord_rpc.initialize('1221506688276893776', callbacks=callbacks, log=False)
                    discord_rpc.update_presence(**presence)
                    discord_rpc.update_connection()
                    time.sleep(.1)
                    discord_rpc.run_callbacks()
                    time.sleep(.2)
                else:
                    discord_rpc.shutdown()
                    time.sleep(3)
            except Exception as exception:
                print(exception)
                time.sleep(1)
                print("Reload DiscordRPC////////////////")
                time.sleep(10)
                continue
    renpy.invoke_in_thread(DiscordRPC)


init:
    python:
    #mod menu game time
        def show_gametime(st, at):
            n = int(renpy.get_game_runtime())
            minutes, seconds = divmod(n, 60)
            hours, minutes = divmod(minutes, 60)
            days, hours = divmod(hours, 24)
            years, days = divmod(days, 365)
            img = Text("Time spent in the game: %0*d:%0*d:%0*d:%0*d:%0*d" % (2, years, 3, days, 2, hours, 2, minutes, 2, seconds))
            return img, None

    image gametime = DynamicDisplayable(show_gametime)